class Drone {
  constructor(globe) {
    this.globe = globe
    this.speed = 0.01 + Math.random() * 0.2 + globe.droneSpeedMultiplier
    this.mesh = this.createDroneMesh()
    this.position = this.getRandomPosition()
    this.target = this.getRandomPosition()
    this.mesh.position.copy(this.position)
    this.isExploding = false
    this.explosionProgress = 0
    this.explosionDuration = 1000 // 1 second explosion animation

    globe.scene.add(this.mesh)
  }

  createDroneMesh() {
    // Create drone body
    const bodyGeometry = new THREE.BoxGeometry(0.03, 0.01, 0.02)
    const bodyMaterial = new THREE.MeshPhongMaterial({
      color: 0xff4444,
      emissive: 0x441111,
      shininess: 30,
    })
    const body = new THREE.Mesh(bodyGeometry, bodyMaterial)

    // Create drone wings
    const wingGeometry = new THREE.BoxGeometry(0.02, 0.002, 0.02)
    const wingMaterial = new THREE.MeshPhongMaterial({
      color: 0x666666,
      shininess: 20,
    })

    const leftWing = new THREE.Mesh(wingGeometry, wingMaterial)
    leftWing.position.x = -0.02
    const rightWing = new THREE.Mesh(wingGeometry, wingMaterial)
    rightWing.position.x = 0.02

    // Create group to hold all drone parts
    const droneGroup = new THREE.Group()
    droneGroup.add(body)
    droneGroup.add(leftWing)
    droneGroup.add(rightWing)

    return droneGroup
  }

  getRandomPosition() {
    const radius = 0.8 // Slightly larger than Earth's radius
    const theta = Math.random() * Math.PI * 2
    const phi = Math.random() * Math.PI

    return new THREE.Vector3(
      radius * Math.sin(phi) * Math.cos(theta),
      radius * Math.sin(phi) * Math.sin(theta),
      radius * Math.cos(phi),
    )
  }

  update(delta) {
    if (this.globe.gameOver) return
    if (this.isExploding) {
      this.updateExplosion(delta)
      return
    }

    // Move towards target
    const direction = this.target.clone().sub(this.position).normalize()
    this.position.add(direction.multiplyScalar(this.speed * delta))
    this.mesh.position.copy(this.position)

    // Look in the direction of movement
    this.mesh.lookAt(this.target)

    // If close to target, get new target
    if (this.position.distanceTo(this.target) < 0.1) {
      this.target = this.getRandomPosition()
    }

    // Check for wave collisions
    this.checkWaveCollisions()
  }

  // Update the Drone class's checkWaveCollisions method
  checkWaveCollisions() {
    // Check all active waves
    this.globe.activeWaves.forEach((wave) => {
      const waveRadius = wave.mesh.scale.x * 0.5
      const distanceFromCenter = this.position.length()
      if (
        Math.abs(distanceFromCenter - waveRadius) < 0.05 &&
        !this.isExploding
      ) {
        this.startExplosion()
      }
    })
  }

  startExplosion() {
    this.isExploding = true
    this.explosionStartTime = Date.now()
    // Increment score when drone is destroyed
    this.globe.score++
    document.getElementById("score-display").textContent =
      `Drones Destroyed: ${this.globe.score}`

    // Create explosion particles
    const particleCount = 20
    const particles = new THREE.Group()

    for (let i = 0; i < particleCount; i++) {
      const geometry = new THREE.BoxGeometry(0.01, 0.01, 0.01)
      const material = new THREE.MeshPhongMaterial({
        color: Math.random() > 0.5 ? 0xff4444 : 0xff7700,
        emissive: 0x441111,
      })
      const particle = new THREE.Mesh(geometry, material)

      // Random direction for particle
      particle.velocity = new THREE.Vector3(
        (Math.random() - 0.5) * 0.02,
        (Math.random() - 0.5) * 0.02,
        (Math.random() - 0.5) * 0.02,
      )

      particles.add(particle)
    }

    this.globe.scene.add(particles)
    this.explosionParticles = particles
  }
  // Modify the Drone class's updateExplosion method
  updateExplosion(delta) {
    const elapsed = Date.now() - this.explosionStartTime
    const progress = elapsed / this.explosionDuration

    if (progress >= 1) {
      // Remove drone and particles
      this.globe.scene.remove(this.mesh)
      this.globe.scene.remove(this.explosionParticles)
      this.globe.drones = this.globe.drones.filter((d) => d !== this)

      return
    }

    // Update particle positions and scale
    this.explosionParticles.children.forEach((particle) => {
      particle.position.add(particle.velocity)
      particle.scale.multiplyScalar(0.95)
    })

    // Fade out original drone
    this.mesh.scale.multiplyScalar(0.9)
  }
}

class Globe {
  constructor() {
    // Keep only the essential properties that shouldn't be reset
    this.renderer = null
    this.scene = null
    this.camera = null
    this.animationId = null
    this.gameOverScreen = null
    this.empButton = null
    this.soundButton = null

    // Basic initialization
    this.createScoreDisplay()
    this.isDragging = false
    this.previousMousePosition = { x: 0, y: 0 }
    this.shouldRotate = true
  }

  setupGame() {
    // Game state
    this.earth = null
    this.spikes = []
    this.stars = null
    this.earthHealth = 1.0
    this.gameOver = false
    this.score = 0

    // Camera settings
    this.cameraDistance = 1
    this.cameraRotation = {
      x: Math.PI * 0.1,
      y: Math.PI * 0.6,
    }
    this.minZoom = 0.1
    this.maxZoom = 10
    this.currentZoom = 0.7
    this.zoomSpeed = 0.02

    // Wave and weapon settings
    this.activeWaves = []
    this.cooldownDuration = 1000

    // Drone settings
    this.drones = []
    this.droneSpeedMultiplier = 0
    this.lastDroneSpawn = 0
    this.droneSpawnInterval = 1000
    this.maxDrones = 200

    // Initialize scene, camera, renderer
    this.scene = new THREE.Scene()
    const height = window.innerHeight - 100
    this.camera = new THREE.PerspectiveCamera(
      75,
      window.innerWidth / height,
      0.1,
      1000,
    )
    this.updateCameraPosition()

    // Set up renderer
    this.renderer = new THREE.WebGLRenderer({ antialias: true })
    this.renderer.setSize(window.innerWidth, height)
    this.renderer.setClearColor(0x000000)
    document.body.prepend(this.renderer.domElement)

    // Create game elements
    this.createStarField()
    this.createEarth()
    this.setupLighting()

    // Set up controls
    this.setupMouseControls()
    this.setupZoomControls()

    // Reset score display
    document.getElementById("score-display").textContent = "Drones Destroyed: 0"

    // Start animation
    this.animate()
  }

  createEarth() {
    const geometry = new THREE.SphereGeometry(0.5, 32, 32)
    const texture = new THREE.TextureLoader().load("earth_atmos_2048.jpg")
    const material = new THREE.MeshPhongMaterial({
      map: texture,
      specular: 0x333333,
      shininess: 5,
      emissive: 0xff0000,
      emissiveIntensity: 0,
    })

    this.earth = new THREE.Mesh(geometry, material)
    this.scene.add(this.earth)
  }

  setupLighting() {
    const ambientLight = new THREE.AmbientLight(0xfffffff)
    this.scene.add(ambientLight)

    const directionalLight = new THREE.DirectionalLight(0xffffff, 0.4)
    directionalLight.position.set(5, 3, 5)
    this.scene.add(directionalLight)
  }

  createGlobe() {
    this.removeGlobe()
    this.setupGame()
    return this
  }

  createScoreDisplay() {
    const scoreDisplay = document.createElement("div")
    scoreDisplay.style.position = "fixed"
    scoreDisplay.style.top = "20px"
    scoreDisplay.style.right = "20px"
    scoreDisplay.style.padding = "10px 20px"
    scoreDisplay.style.backgroundColor = "rgba(0, 0, 0, 0.8)"
    scoreDisplay.style.color = "#4CAF50"
    scoreDisplay.style.borderRadius = "10px"
    scoreDisplay.style.fontSize = "24px"
    scoreDisplay.style.fontFamily = "monospace"
    scoreDisplay.style.zIndex = "1000"
    scoreDisplay.style.border = "1px solid rgba(76, 175, 80, 0.3)"
    scoreDisplay.id = "score-display"
    scoreDisplay.textContent = "Drones Destroyed: 0"
    document.body.appendChild(scoreDisplay)
  }

  createControlPanel() {
    const controlPanel = document.createElement("div")
    controlPanel.style.position = "fixed"
    controlPanel.style.bottom = "20px"
    controlPanel.style.right = "20px"
    controlPanel.style.padding = "20px"
    controlPanel.style.backgroundColor = "rgba(0, 0, 0, 0.8)"
    controlPanel.style.borderRadius = "15px"
    controlPanel.style.zIndex = "1000"
    controlPanel.style.display = "flex"
    controlPanel.style.flexDirection = "column"
    controlPanel.style.alignItems = "center"
    controlPanel.style.gap = "10px"

    // Add styles for the cooldown animation
    const style = document.createElement("style")
    style.textContent = `
      .control-button {
        width: 40px;
        height: 40px;
        border: none;
        border-radius: 50%;
        cursor: pointer;
        font-size: 16px;
        font-weight: bold;
        color: white;
        position: relative;
        transition: transform 0.5s ease;
      }
      
      .control-button:not(.cooldown):hover {
        transform: scale(1.1);
      }
      
      .control-button.cooldown {
        cursor: not-allowed;
        background-color: transparent !important;
      }
    `
    document.head.appendChild(style)

    const buttonContainer = document.createElement("div")
    buttonContainer.style.display = "flex"
    buttonContainer.style.gap = "15px"
    buttonContainer.style.position = "relative"
    buttonContainer.style.zIndex = "1"

    const createButton = (text, color, action) => {
      const button = document.createElement("button")
      button.textContent = text
      button.className = "control-button"
      button.style.setProperty("--button-color", color)
      button.style.backgroundColor = color

      button.addEventListener("click", () => this.handleAction(action))
      return button
    }

    // Create buttons with their base colors
    const empButton = createButton("E", "#4CAF50", "EMP")
    const soundButton = createButton("S", "#2196F3", "SOUND")

    // Store button references
    this.empButton = empButton
    this.soundButton = soundButton

    buttonContainer.appendChild(empButton)
    buttonContainer.appendChild(soundButton)
    controlPanel.appendChild(buttonContainer)

    // Add keyboard shortcut hint
    const shortcutHint = document.createElement("div")
    shortcutHint.textContent = "Press E or S"
    shortcutHint.style.color = "rgba(255, 255, 255, 0.5)"
    shortcutHint.style.fontSize = "12px"
    shortcutHint.style.marginTop = "5px"
    controlPanel.appendChild(shortcutHint)

    document.body.appendChild(controlPanel)

    // Add keyboard listeners
    document.addEventListener("keydown", (e) => {
      if (e.key.toLowerCase() === "e") {
        this.handleAction("EMP")
      } else if (e.key.toLowerCase() === " ") {
        this.handleAction("EMP")
      } else if (e.key.toLowerCase() === "s") {
        this.handleAction("SOUND")
      } else if (e.key.toLowerCase() === "f") {
        this.toggleFullScreen()
      }
    })
  }

  handleAction(action) {
    const cooldownProperty = action.toLowerCase() + "Cooldown"
    const button = action === "EMP" ? this.empButton : this.soundButton

    if (!this[cooldownProperty]) {
      if (action === "EMP") {
        this.triggerEMP()
      } else {
        this.triggerSound()
      }

      // Handle cooldown and animation
      this[cooldownProperty] = true
      button.classList.add("cooldown")

      setTimeout(() => {
        this[cooldownProperty] = false
        button.classList.remove("cooldown")
      }, this.cooldownDuration)
    }
  }
  triggerEMP() {
    const wave = this.triggerWave({
      type: "emp",
      color: 0x00ff00,
      duration: 2000,
      maxScale: 10,
      pulseEffect: false,
    })
    this.activeWaves.push(wave)
  }

  triggerSound() {
    const wave = this.triggerWave({
      type: "sound",
      color: 0x2196f3,
      duration: 4000,
      maxScale: 8,
      pulseEffect: true,
    })
    this.activeWaves.push(wave)
  }

  triggerWave(config) {
    const {
      type,
      color = 0x00ff00,
      duration = 2000,
      maxScale = 10,
      pulseEffect = false,
    } = config

    const waveGeometry = new THREE.SphereGeometry(0.5, 32, 32)
    const waveMaterial = new THREE.ShaderMaterial({
      transparent: true,
      uniforms: {
        time: { value: 0 },
        color: { value: new THREE.Color(color) },
        maxRadius: { value: 5.0 },
      },
      vertexShader: `
      varying vec3 vNormal;
      void main() {
        vNormal = normalize(normalMatrix * normal);
        gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
      }
    `,
      fragmentShader: `
      uniform float time;
      uniform vec3 color;
      uniform float maxRadius;
      varying vec3 vNormal;

      void main() {
        float intensity = 1.0 - (time * time);
        float edge = 0.5;
        float rim = smoothstep(0.5 - edge, 0.5 + edge, dot(vNormal, vec3(0.0, 0.0, 1.0)));
        ${pulseEffect ? "float wave = sin(time * 20.0) * 0.5 + 0.5;" : "float wave = 1.0;"}
        gl_FragColor = vec4(color, intensity * (1.0 - rim) * wave * 0.7);
      }
    `,
    })

    const waveMesh = new THREE.Mesh(waveGeometry, waveMaterial)
    this.scene.add(waveMesh)

    const wave = {
      type,
      mesh: waveMesh,
      geometry: waveGeometry,
      material: waveMaterial,
      startTime: Date.now(),
      duration,
      maxScale,
    }

    // Add to active waves and return
    this.activeWaves.push(wave)
    return wave
  }

  updateCameraPosition() {
    // Convert spherical coordinates to Cartesian
    const phi = this.cameraRotation.x // vertical angle
    const theta = this.cameraRotation.y // horizontal angle
    const radius = this.cameraDistance / this.currentZoom

    this.camera.position.x = radius * Math.cos(phi) * Math.cos(theta)
    this.camera.position.y = radius * Math.sin(phi)
    this.camera.position.z = radius * Math.cos(phi) * Math.sin(theta)

    // Always look at the center
    this.camera.lookAt(0, 0, 0)
    this.camera.up.set(0, 1, 0) // Keep "up" direction consistent
  }

  createStarField() {
    const starsGeometry = new THREE.BufferGeometry()
    const starsMaterial = new THREE.PointsMaterial({
      color: 0xffffff,
      size: 0.02,
      transparent: true,
      opacity: 0.8,
      sizeAttenuation: true,
    })

    const starsVertices = []
    const radius = 10 // Radius of our star sphere
    const starsCount = 2000 // Number of stars

    for (let i = 0; i < starsCount; i++) {
      // Create random spherical coordinates
      const theta = 2 * Math.PI * Math.random()
      const phi = Math.acos(2 * Math.random() - 1)
      const x = radius * Math.sin(phi) * Math.cos(theta)
      const y = radius * Math.sin(phi) * Math.sin(theta)
      const z = radius * Math.cos(phi)

      starsVertices.push(x, y, z)
    }

    starsGeometry.setAttribute(
      "position",
      new THREE.Float32BufferAttribute(starsVertices, 3),
    )

    this.stars = new THREE.Points(starsGeometry, starsMaterial)
    this.scene.add(this.stars)
  }

  damageEarth() {
    if (this.gameOver) return

    this.earthHealth = Math.max(0, this.earthHealth - 0.1) // Reduce health by 10%
    this.earth.material.emissiveIntensity = 1 - this.earthHealth // Increase red glow

    if (this.earthHealth <= 0) {
      this.triggerGameOver()
    }
  }

  triggerGameOver() {
    this.gameOver = true

    // Create explosion particles
    const particleCount = 1000
    const particles = new THREE.Group()

    for (let i = 0; i < particleCount; i++) {
      const geometry = new THREE.BoxGeometry(0.02, 0.02, 0.02)
      const material = new THREE.MeshPhongMaterial({
        color: Math.random() > 0.5 ? 0xff4444 : 0xff7700,
        emissive: 0x441111,
      })
      const particle = new THREE.Mesh(geometry, material)

      // Position particles on earth's surface
      const theta = Math.random() * Math.PI * 2
      const phi = Math.random() * Math.PI
      const radius = 0.5

      particle.position.x = radius * Math.sin(phi) * Math.cos(theta)
      particle.position.y = radius * Math.sin(phi) * Math.sin(theta)
      particle.position.z = radius * Math.cos(phi)

      // Add velocity for explosion
      const velocity = new THREE.Vector3()
        .copy(particle.position)
        .normalize()
        .multiplyScalar(0.03)
      particle.velocity = velocity

      particles.add(particle)
    }

    this.scene.add(particles)
    this.explosionParticles = particles

    // Hide Earth
    this.earth.visible = false

    // Create game over screen
    this.createGameOverScreen()

    // Start explosion animation
    this.animateExplosion()
  }

  createGameOverScreen() {
    // Create container for game over screen
    const gameOverScreen = document.createElement("div")
    gameOverScreen.style.position = "fixed"
    gameOverScreen.style.top = "50%"
    gameOverScreen.style.left = "50%"
    gameOverScreen.style.transform = "translate(-50%, -50%)"
    gameOverScreen.style.textAlign = "center"
    gameOverScreen.style.color = "#ff0000"
    gameOverScreen.style.fontFamily = "Arial, sans-serif"
    gameOverScreen.style.zIndex = "1000"

    // Game Over text
    const gameOverText = document.createElement("div")
    gameOverText.textContent = "GAME OVER"
    gameOverText.style.fontSize = "64px"
    gameOverText.style.fontWeight = "bold"
    gameOverText.style.textShadow = "0 0 10px #ff0000"
    gameOverText.style.marginBottom = "20px"

    // Score text
    const scoreText = document.createElement("div")
    scoreText.textContent = `Score: ${this.score}`
    scoreText.style.fontSize = "32px"
    scoreText.style.marginBottom = "40px"

    // Play Again button
    const playAgainButton = document.createElement("button")
    playAgainButton.textContent = "Play Again"
    playAgainButton.style.padding = "15px 30px"
    playAgainButton.style.fontSize = "24px"
    playAgainButton.style.backgroundColor = "#4CAF50"
    playAgainButton.style.color = "white"
    playAgainButton.style.border = "none"
    playAgainButton.style.borderRadius = "5px"
    playAgainButton.style.cursor = "pointer"
    playAgainButton.style.transition = "background-color 0.3s"

    // Hover effect
    playAgainButton.addEventListener("mouseover", () => {
      playAgainButton.style.backgroundColor = "#45a049"
    })
    playAgainButton.addEventListener("mouseout", () => {
      playAgainButton.style.backgroundColor = "#4CAF50"
    })

    // Click event
    playAgainButton.addEventListener("click", () => this.resetGame())

    // Add elements to game over screen
    gameOverScreen.appendChild(gameOverText)
    gameOverScreen.appendChild(scoreText)
    gameOverScreen.appendChild(playAgainButton)

    // Store reference and add to document
    this.gameOverScreen = gameOverScreen
    document.body.appendChild(gameOverScreen)
  }

  resetGame() {
    // Remove existing elements
    this.removeGlobe()

    if (this.gameOverScreen) {
      this.gameOverScreen.remove()
      this.gameOverScreen = null
    }

    // Reset buttons
    if (this.empButton) {
      this.empButton.classList.remove("cooldown")
      this.empButton.style.cursor = "pointer"
      this.empButton.style.opacity = "1"
    }
    if (this.soundButton) {
      this.soundButton.classList.remove("cooldown")
      this.soundButton.style.cursor = "pointer"
      this.soundButton.style.opacity = "1"
    }

    // Reinitialize game
    this.setupGame()
  }

  animateExplosion() {
    if (!this.explosionParticles) return

    this.explosionParticles.children.forEach((particle) => {
      particle.position.add(particle.velocity)
      particle.scale.multiplyScalar(0.98)
    })

    requestAnimationFrame(() => this.animateExplosion())
  }

  setupMouseControls() {
    const canvas = this.renderer.domElement

    canvas.addEventListener("mousedown", (e) => {
      this.isDragging = true
      this.previousMousePosition = {
        x: e.clientX,
        y: e.clientY,
      }
    })

    canvas.addEventListener("mousemove", (e) => {
      if (!this.isDragging) return

      const deltaMove = {
        x: e.clientX - this.previousMousePosition.x,
        y: e.clientY - this.previousMousePosition.y,
      }

      // Adjust rotation speed based on movement
      const rotationSpeed = 0.005

      // Update camera orbit angles
      this.cameraRotation.y += deltaMove.x * rotationSpeed
      this.cameraRotation.x += deltaMove.y * rotationSpeed

      // Limit vertical rotation to prevent flipping
      this.cameraRotation.x = Math.max(
        -Math.PI / 2 + 0.1,
        Math.min(Math.PI / 2 - 0.1, this.cameraRotation.x),
      )

      // Update camera position
      this.updateCameraPosition()

      this.previousMousePosition = {
        x: e.clientX,
        y: e.clientY,
      }
    })

    window.addEventListener("mouseup", () => {
      this.isDragging = false
    })

    canvas.addEventListener("selectstart", (e) => {
      e.preventDefault()
    })
  }

  setupZoomControls() {
    const canvas = this.renderer.domElement

    canvas.addEventListener("wheel", (e) => {
      e.preventDefault()

      const zoomDelta = e.deltaY > 0 ? -this.zoomSpeed : this.zoomSpeed

      this.currentZoom = Math.max(
        this.minZoom,
        Math.min(this.maxZoom, this.currentZoom + zoomDelta),
      )

      this.updateCameraPosition()
    })

    let touchDistance = 0

    canvas.addEventListener("touchstart", (e) => {
      if (e.touches.length === 2) {
        const touch1 = e.touches[0]
        const touch2 = e.touches[1]
        touchDistance = Math.hypot(
          touch2.clientX - touch1.clientX,
          touch2.clientY - touch1.clientY,
        )
      }
    })

    canvas.addEventListener("touchmove", (e) => {
      if (e.touches.length === 2) {
        e.preventDefault()

        const touch1 = e.touches[0]
        const touch2 = e.touches[1]
        const newDistance = Math.hypot(
          touch2.clientX - touch1.clientX,
          touch2.clientY - touch1.clientY,
        )

        const delta = newDistance - touchDistance
        const zoomDelta = delta * 0.01 * this.zoomSpeed

        this.currentZoom = Math.max(
          this.minZoom,
          Math.min(this.maxZoom, this.currentZoom + zoomDelta),
        )

        this.updateCameraPosition()
        touchDistance = newDistance
      }
    })
  }

  animate() {
    if (!this.earth) return

    const { earth, spikes, renderer, scene, camera } = this

    this.animationId = requestAnimationFrame(this.animate.bind(this))

    const currentTime = Date.now()
    const delta = (currentTime - (this.lastFrameTime || currentTime)) / 1000
    this.lastFrameTime = currentTime

    // Update all waves in the main animation loop
    this.activeWaves = this.activeWaves.filter((wave) => {
      const elapsed = currentTime - wave.startTime
      const progress = elapsed / wave.duration

      if (progress >= 1) {
        this.scene.remove(wave.mesh)
        wave.geometry.dispose()
        wave.material.dispose()
        return false
      }

      const scale = 1 + (wave.maxScale - 1) * progress
      wave.mesh.scale.set(scale, scale, scale)
      wave.material.uniforms.time.value = progress
      return true
    })

    // Spawn new drones
    if (
      currentTime - this.lastDroneSpawn > this.droneSpawnInterval &&
      this.drones.length < this.maxDrones &&
      !this.gameOver
    ) {
      this.drones.push(new Drone(this))
      this.lastDroneSpawn = currentTime
      this.droneSpawnInterval = Math.max(10, this.droneSpawnInterval * 0.95)
      this.droneSpeedMultiplier += 0.001
    }

    // Check for drone collisions with Earth
    this.drones.forEach((drone) => {
      if (!drone.isExploding) {
        const distanceFromCenter = drone.position.length()
        if (distanceFromCenter <= 0.52) {
          // Slightly larger than Earth's radius (0.5)
          this.damageEarth()
          drone.startExplosion()
        }
      }
    })

    // Update drones
    this.drones.forEach((drone) => drone.update(delta))

    const AUTO_ROTATE_SPEED = 0.0003

    // Auto-rotate camera when not dragging
    if (this.shouldRotate && !this.isDragging) {
      this.cameraRotation.y += AUTO_ROTATE_SPEED
      this.updateCameraPosition()
    }

    // Update spikes
    spikes.forEach((spike, index) => {
      spike.scale.y *= 0.95
      if (spike.scale.y < 0.01) {
        earth.remove(spike)
        spikes.splice(index, 1)
      }
    })

    renderer.render(scene, camera)
  }

  removeGlobe() {
    // Cancel the previous animation frame to stop multiple animations
    if (this.animationId) {
      cancelAnimationFrame(this.animationId)
    }

    // Remove event listeners if they exist
    if (this.renderer) {
      const canvas = this.renderer.domElement
      canvas.removeEventListener("mousedown", this.onMouseDown)
      canvas.removeEventListener("mousemove", this.onMouseMove)
      canvas.removeEventListener("selectstart", this.onSelectStart)
    }

    // Dispose of the renderer, scene, and event listeners
    if (this.renderer) {
      this.renderer.dispose()
      this.renderer.domElement.remove()
    }
    if (this.scene) {
      while (this.scene.children.length > 0) {
        const object = this.scene.children[0]
        if (object.geometry) object.geometry.dispose()
        if (object.material) object.material.dispose()
        this.scene.remove(object)
      }
    }
    this.spikes = []
    this.renderer = null
    this.scene = null
    this.earth = null
  }

  latLongToVector3(lat, lon) {
    const phi = (90 - lat) * (Math.PI / 180)
    const theta = (lon + 180) * (Math.PI / 180)
    const x = -0.5 * Math.sin(phi) * Math.cos(theta)
    const y = 0.5 * Math.cos(phi)
    const z = 0.5 * Math.sin(phi) * Math.sin(theta)
    return new THREE.Vector3(x, y, z)
  }

  visualizeHit(request) {
    const { lat, long, type } = request
    const position = this.latLongToVector3(lat, long)

    const spikeGeometry = new THREE.ConeGeometry(0.005, 0.3, 8)
    spikeGeometry.translate(0, 0.05, 0)
    spikeGeometry.rotateX(Math.PI / 2)

    const color = type === "read" ? 0xffffff : 0x00ff00
    const spikeMaterial = new THREE.MeshBasicMaterial({ color })

    const spike = new THREE.Mesh(spikeGeometry, spikeMaterial)
    spike.position.set(position.x, position.y, position.z)
    spike.lookAt(new THREE.Vector3(0, 0, 0))

    this.earth.add(spike) // Add spike to earth instead of scene
    this.spikes.push(spike)
  }

  listenToResize() {
    window.addEventListener("resize", () => {
      this.createGlobe() // Recreate the globe on resize
    })
    return this
  }

  toggleFullScreen() {
    if (!document.fullscreenElement) {
      document.body.requestFullscreen().catch((err) => {
        console.log(
          `Error trying to go fullscreen: ${err.message} (${err.name})`,
        )
      })
    } else {
      document.exitFullscreen()
    }
  }

  shouldRotate = true
}

window.globe = new Globe().createGlobe().listenToResize()

// Add the control panel after initialization
window.globe.createControlPanel()
